@@ -1,37 +1,38 @@
+from __future__ import print_function, division
+
 # Need to depracate fits_read, etc.
 
+import string
 import warnings
-import atpy
+from copy import deepcopy
 
 import numpy as np
 import numpy.ma as ma
 
-from copy import deepcopy
-import string
-
-from exceptions import VectorException
-
-import structhelper as sta
-from odict import odict
+from .exceptions import VectorException
+from .structhelper import append_field, drop_fields
+from .odict import odict
+from . import registry
+from .masked import __masked__
 
 default_format = {}
-default_format[None.__class__] = 16, '.9e'
-default_format[np.bool_] = 1, 'i'
-default_format[np.int8] = 3, 'i'
-default_format[np.uint8] = 3, 'i'
-default_format[np.int16] = 5, 'i'
-default_format[np.uint16] = 5, 'i'
-default_format[np.int32] = 12, 'i'
-default_format[np.uint32] = 12, 'i'
-default_format[np.int64] = 22, 'i'
-default_format[np.uint64] = 23, 'i'
-default_format[np.float32] = 16, '.8e'
-default_format[np.float64] = 25, '.17e'
-default_format[np.str] = 0, 's'
-default_format[np.string_] = 0, 's'
-default_format[np.uint8] = 0, 's'
-default_format[str] = 0, 's'
-default_format[np.unicode_] = 0, 's'
+default_format[None.__class__] = '16.9e'
+default_format[np.bool_] = '1i'
+default_format[np.int8] = '3i'
+default_format[np.uint8] = '3i'
+default_format[np.int16] = '5i'
+default_format[np.uint16] = '5i'
+default_format[np.int32] = '12i'
+default_format[np.uint32] = '12i'
+default_format[np.int64] = '22i'
+default_format[np.uint64] = '23i'
+default_format[np.float32] = '16.8e'
+default_format[np.float64] = '25.17e'
+default_format[np.str] = 's'
+default_format[np.string_] = 's'
+default_format[np.uint8] = 's'
+default_format[str] = 's'
+default_format[np.unicode_] = 's'
 
 
 class ColumnHeader(object):
@@ -58,23 +59,23 @@ class ColumnHeader(object):
         if self.null:
             s += ", null=%s" % str(self.null)
         if self.description:
-            s +=", description=%s" % self.description
+            s += ", description=%s" % self.description
         return s
 
     def __eq__(self, other):
-        if self.dtype <> other.dtype:
+        if self.dtype != other.dtype:
             return False
-        if self.unit <> other.unit:
+        if self.unit != other.unit:
             return False
-        if self.description <> other.description:
+        if self.description != other.description:
             return False
-        if self.null <> other.null:
+        if self.null != other.null:
             if np.isnan(self.null):
                 if not np.isnan(other.null):
                     return False
             else:
                 return False
-        if self.format <> other.format:
+        if self.format != other.format:
             return False
         return True
 
@@ -161,7 +162,7 @@ class Table(object):
         if 'masked' in kwargs:
             self._masked = kwargs.pop('masked')
         else:
-            self._masked = atpy.__masked__
+            self._masked = __masked__
 
         if len(args) + len(kwargs) > 0:
             self.read(*args, **kwargs)
@@ -190,10 +191,10 @@ class Table(object):
 
         if 'type' in kwargs:
             table_type = kwargs.pop('type').lower()
-        elif type(args[0]) == str:
-            table_type = atpy._determine_type(args[0], verbose)
+        elif isinstance(args[0], basestring):
+            table_type = registry._determine_type(args[0], verbose)
         else:
-            raise Exception('Could not determine input type')
+            raise Exception('Could not determine table type')
 
         original_filters = warnings.filters[:]
 
@@ -209,8 +210,8 @@ class Table(object):
             else:
                 warnings.simplefilter("ignore")
 
-            if table_type in atpy._readers:
-                atpy._readers[table_type](self, *args, **kwargs)
+            if table_type in registry._readers:
+                registry._readers[table_type](self, *args, **kwargs)
             else:
                 raise Exception("Unknown table type: " + table_type)
 
@@ -242,9 +243,9 @@ class Table(object):
         if 'type' in kwargs:
             table_type = kwargs.pop('type').lower()
         elif type(args[0]) == str:
-            table_type = atpy._determine_type(args[0], verbose)
+            table_type = registry._determine_type(args[0], verbose)
         else:
-            raise Exception('Could not determine input type')
+            raise Exception('Could not determine table type')
 
         original_filters = warnings.filters[:]
 
@@ -255,8 +256,8 @@ class Table(object):
 
         try:
 
-            if table_type in atpy._writers:
-                atpy._writers[table_type](self, *args, **kwargs)
+            if table_type in registry._writers:
+                registry._writers[table_type](self, *args, **kwargs)
             else:
                 raise Exception("Unknown table type: " + table_type)
 
@@ -270,16 +271,16 @@ class Table(object):
         if attribute == 'names':
             return self.__dict__['data'].dtype.names
         elif attribute == 'units':
-            print "WARNING: Table.units is depracated - use Table.columns to access this information"
+            print("WARNING: Table.units is deprecated - use Table.columns to access this information")
             return dict([(name, self.columns[name].unit) for name in self.names])
         elif attribute == 'types':
-            print "WARNING: Table.types is depracated - use Table.columns to access this information"
+            print("WARNING: Table.types is deprecated - use Table.columns to access this information")
             return dict([(name, self.columns[name].type) for name in self.names])
         elif attribute == 'nulls':
-            print "WARNING: Table.nulls is depracated - use Table.columns to access this information"
+            print("WARNING: Table.nulls is deprecated - use Table.columns to access this information")
             return dict([(name, self.columns[name].null) for name in self.names])
         elif attribute == 'formats':
-            print "WARNING: Table.formats is depracated - use Table.columns to access this information"
+            print("WARNING: Table.formats is deprecated - use Table.columns to access this information")
             return dict([(name, self.columns[name].format) for name in self.names])
         elif attribute == 'shape':
             return (len(self.data), len(self.names))
@@ -292,13 +293,23 @@ class Table(object):
     def __getitem__(self, item):
         return self.data[item]
 
+    def __setitem__(self, item, value):
+        if 'data' in self.__dict__:
+            if isinstance(self.data, np.ndarray):
+                if item in self.data.dtype.names:
+                    self.data[item] = value
+                    return
+        raise ValueError("Column %s does not exist" % item)
+
     def keys(self):
         return self.data.dtype.names
 
     def append(self, table):
         for colname in self.columns:
-            if self.columns[colname] <> table.columns[colname]:
-                raise Exception("Columns do not match")
+            if self.columns.keys != table.columns.keys:
+                raise Exception("Column names do not match")
+            if self.columns[colname].dtype.type != table.columns[colname].dtype.type:
+                raise Exception("Column types do not match")
         self.data = np.hstack((self.data, table.data))
 
     def __setattr__(self, attribute, value):
@@ -319,7 +330,7 @@ class Table(object):
         '''
         Empty the table
         '''
-        self.keywords = {}
+        self.keywords = odict()
         self.comments = []
         self.columns = odict()
         self.data = None
@@ -355,7 +366,7 @@ class Table(object):
             else:
                 description = descriptions[i]
 
-            if formats is None:
+            if formats is None or format in ['e', 'g', 'f']:
                 if dtype[i].subdtype:
                     format = default_format[dtype[i].subdtype[0].type]
                 else:
@@ -363,8 +374,12 @@ class Table(object):
             else:
                 format = formats[i]
 
-            if format[1] == 's':
-                format = self.data.itemsize, 's'
+            # Backward compatibility with tuple-style format
+            if type(format) in [tuple, list]:
+                format = string.join([str(x) for x in format], "")
+
+            if format == 's':
+                format = '%is' % self.data.itemsize
 
             if nulls is None:
                 null = None
@@ -376,7 +391,7 @@ class Table(object):
             self.columns[dtype.names[i]] = column
 
     def add_empty_column(self, name, dtype, unit='', null='', \
-        description='', format=None, shape=None, before=None, after=None, \
+        description='', format=None, column_header=None, shape=None, before=None, after=None, \
         position=None):
         '''
         Add an empty column to the table. This only works if there
@@ -405,6 +420,14 @@ class Table(object):
             *format*: [ string ]
                 The format to use for ASCII printing
 
+            *column_header*: [ ColumnHeader ]
+                The metadata from an existing column to copy over. Metadata
+                includes the unit, null value, description, format, and dtype.
+                For example, to create a column 'b' with identical metadata to
+                column 'a' in table 't', use:
+
+                    >>> t.add_column('b', column_header=t.columns[a])
+
             *shape*: [ tuple ]
                 Tuple describing the shape of the empty column that is to be
                 added. If a one element tuple is specified, it is the number
@@ -429,11 +452,11 @@ class Table(object):
             raise Exception("Table is empty, you need to use the shape= argument to specify the dimensions of the first column")
 
         self.add_column(name, data, unit=unit, null=null, \
-            description=description, format=format, before=before, \
+            description=description, format=format, column_header=column_header, before=before, \
             after=after, position=position)
 
     def add_column(self, name, data, unit='', null='', description='', \
-        format=None, dtype=None, before=None, after=None, position=None, mask=None, fill=None):
+        format=None, dtype=None, column_header=None, before=None, after=None, position=None, mask=None, fill=None):
         '''
         Add a column to the table
 
@@ -463,6 +486,14 @@ class Table(object):
                 Numpy type to convert the data to. This is the equivalent to
                 the dtype= argument in numpy.array
 
+            *column_header*: [ ColumnHeader ]
+                The metadata from an existing column to copy over. Metadata
+                includes the unit, null value, description, format, and dtype.
+                For example, to create a column 'b' with identical metadata to
+                column 'a' in table 't', use:
+
+                    >>> t.add_column('b', column_header=t.columns[a])
+
             *before*: [ string ]
                 Column before which the new column should be inserted
 
@@ -482,6 +513,28 @@ class Table(object):
                 value in the numpy masked array.
         '''
 
+        if column_header is not None:
+
+            if dtype is not None:
+                warnings.warn("dtype= argument overriden by column_header=")
+            dtype = column_header.dtype
+
+            if unit != '':
+                warnings.warn("unit= argument overriden by column_header=")
+            unit = column_header.unit
+
+            if null != '':
+                warnings.warn("null= argument overriden by column_header=")
+            null = column_header.null
+
+            if description != '':
+                warnings.warn("description= argument overriden by column_header=")
+            description = column_header.description
+
+            if format is not None:
+                warnings.warn("format= argument overriden by column_header=")
+            format = column_header.format
+
         if self._masked:
             if null:
                 warnings.warn("null= argument can only be used if Table does not use masked arrays (ignored)")
@@ -494,15 +547,21 @@ class Table(object):
         dtype = data.dtype
 
         if dtype.type == np.object_:
-            longest = len(max(data, key=len))
+
+            if len(data) == 0:
+                longest = 0
+            else:
+                longest = len(max(data, key=len))
+
             if self._masked:
                 data = ma.array(data, dtype='|%iS' % longest)
             else:
                 data = np.array(data, dtype='|%iS' % longest)
+
             dtype = data.dtype
 
         if data.ndim > 1:
-            newdtype = (name, data.dtype, data.shape[1])
+            newdtype = (name, data.dtype, (data.shape[1],))
         else:
             newdtype = (name, data.dtype)
 
@@ -518,18 +577,22 @@ class Table(object):
                 raise Exception("Column %s does not exist" % before)
 
         if len(self.columns) > 0:
-            self.data = sta.append_field(self.data, data, dtype=newdtype, position=position, masked=self._masked)
+            self.data = append_field(self.data, data, dtype=newdtype, position=position, masked=self._masked)
         else:
             if self._masked:
                 self.data = ma.array(zip(data), dtype=[newdtype], mask=zip(data.mask), fill_value=data.fill_value)
             else:
                 self.data = np.array(zip(data), dtype=[newdtype])
 
-        if not format:
+        if not format or format in ['e', 'g', 'f']:
             format = default_format[dtype.type]
 
-        if format[1] == 's':
-            format = data.itemsize, 's'
+        # Backward compatibility with tuple-style format
+        if type(format) in [tuple, list]:
+            format = string.join([str(x) for x in format], "")
+
+        if format == 's':
+            format = '%is' % data.itemsize
 
         column = ColumnHeader(dtype, unit=unit, description=description, null=null, format=format)
 
@@ -541,7 +604,7 @@ class Table(object):
         return
 
     def remove_column(self, remove_name):
-        print "WARNING: remove_column is depracated - use remove_columns instead"
+        print("WARNING: remove_column is deprecated - use remove_columns instead")
         self.remove_columns([remove_name])
         return
 
@@ -561,7 +624,7 @@ class Table(object):
         for remove_name in remove_names:
             self.columns.pop(remove_name)
 
-        self.data = sta.drop_fields(self.data, remove_names, masked=self._masked)
+        self.data = drop_fields(self.data, remove_names, masked=self._masked)
 
         # Remove primary key if needed
         if self._primary_key in remove_names:
@@ -613,7 +676,7 @@ class Table(object):
 
         # tuple.index was only introduced in Python 2.6, so need to use list()
         pos = list(self.names).index(old_name)
-        self.data.dtype.names = self.names[:pos] + (new_name, ) + self.names[pos+1:]
+        self.data.dtype.names = self.names[:pos] + (new_name, ) + self.names[pos + 1:]
 
         if self._masked:
             self.data.mask.dtype.names = self.data.dtype.names[:]
@@ -631,10 +694,14 @@ class Table(object):
         Prints a description of the table
         '''
 
+        if self.data is None:
+            print("Table is empty")
+            return
+
         if self.table_name:
-            print "Table : " + self.table_name
+            print("Table : " + self.table_name)
         else:
-            print "Table has no name"
+            print("Table has no name")
 
         # Find maximum column widths
         len_name_max, len_unit_max, len_datatype_max, \
@@ -650,22 +717,22 @@ class Table(object):
         # Print out table
 
         format = "| %" + str(len_name_max) + \
-            "s | %" + str(len_unit_max) + \
-            "s | %" + str(len_datatype_max) + \
-            "s | %" + str(len_formats_max) + "s |"
+               "s | %" + str(len_unit_max) + \
+               "s | %" + str(len_datatype_max) + \
+               "s | %" + str(len_formats_max) + "s |"
 
         len_tot = len_name_max + len_unit_max + len_datatype_max + \
             len_formats_max + 13
 
-        print "-"*len_tot
-        print format % ("Name", "Unit", "Type", "Format")
-        print "-"*len_tot
+        print("-" * len_tot)
+        print(format % ("Name", "Unit", "Type", "Format"))
+        print("-" * len_tot)
 
         for name in self.names:
-            print format % (name, str(self.columns[name].unit), \
-                str(self.columns[name].dtype), self.format(name))
+            print(format % (name, str(self.columns[name].unit), \
+                str(self.columns[name].dtype), self.columns[name].format))
 
-        print "-"*len_tot
+        print("-" * len_tot)
 
         return
 
@@ -746,17 +813,6 @@ class Table(object):
 
         return new_table
 
-    def format(self, name):
-        '''
-        Return the ASCII format of a given column
-
-        Required Arguments:
-
-            *name*: [ string ]
-                The column name
-        '''
-        return str(self.columns[name].format[0]) + self.columns[name].format[1]
-
     def add_comment(self, comment):
         '''
         Add a comment to the table
@@ -804,10 +860,10 @@ class Table(object):
         if not key in self.names:
             raise Exception("No such column: %s" % key)
         else:
-            if self.columns[key].null <> '':
+            if self.columns[key].null != '':
                 if np.any(self.data[key] == self.columns[key].null):
                     raise Exception("Primary key column cannot contain null values")
-            elif len(np.unique(self.data[key])) <> len(self.data[key]):
+            elif len(np.unique(self.data[key])) != len(self.data[key]):
                 raise Exception("Primary key column cannot contain duplicate values")
             else:
                 self._primary_key = key
@@ -936,9 +992,9 @@ class TableSet(object):
         if 'type' in kwargs:
             table_type = kwargs.pop('type').lower()
         elif type(args[0]) == str:
-            table_type = atpy._determine_type(args[0], verbose)
+            table_type = registry._determine_type(args[0], verbose)
         else:
-            raise Exception('Could not determine input type')
+            raise Exception('Could not determine table type')
 
         original_filters = warnings.filters[:]
 
@@ -954,8 +1010,8 @@ class TableSet(object):
             else:
                 warnings.simplefilter("ignore")
 
-            if table_type in atpy._set_readers:
-                atpy._set_readers[table_type](self, *args, **kwargs)
+            if table_type in registry._set_readers:
+                registry._set_readers[table_type](self, *args, **kwargs)
             else:
                 raise Exception("Unknown table type: " + table_type)
 
@@ -987,9 +1043,9 @@ class TableSet(object):
         if 'type' in kwargs:
             table_type = kwargs.pop('type').lower()
         elif type(args[0]) == str:
-            table_type = atpy._determine_type(args[0], verbose)
+            table_type = registry._determine_type(args[0], verbose)
         else:
-            raise Exception('Could not determine input type')
+            raise Exception('Could not determine table type')
 
         original_filters = warnings.filters[:]
 
@@ -1000,8 +1056,8 @@ class TableSet(object):
 
         try:
 
-            if table_type in atpy._set_writers:
-                atpy._set_writers[table_type](self, *args, **kwargs)
+            if table_type in registry._set_writers:
+                registry._set_writers[table_type](self, *args, **kwargs)
             else:
                 raise Exception("Unknown table type: " + table_type)
 
